// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
  'cloudera/common/charts/rickshaw/FormatUtils',
  'cloudera/common/charts/rickshaw/Visualizer',
  'underscore'
], function(FormatUtils, Visualizer, _) {

  describe("Visualizer Tests", function() {
    var groupTs = [ {
      data: [{x: 1, y: 1}, {x: 2, y: 2}],
      metadata: {
        label: "Foo"
      }
    }, {
      data: [{x: 2, y: 2}, {x: 3, y: 3}],
      metadata: {
        label: "Bar"
      }
    } ];

    var options = {
      container: "#chartContainer",
      width: 100,
      height: 60
    };

    var dummyGraph = {
      render: function() {},
      configure: function() {},
      onUpdate: function() {},
      append: function() {}
    };

    var dummyAxisX = {
      render: function() {}
    };

    var dummyAxisY = {
      render: function() {},
      element: {}
    };

    beforeEach(function() {
      $('<div id="chartContainer"><div class="chart"></div></div>').appendTo(document.body);
    });

    afterEach(function() {
      $("#chartContainer").remove();
    });

    it("should call graph.render", function() {
      var viz = new Visualizer(options);
      // mock a dummy graph.
      spyOn(viz, "createGraph").andReturn(dummyGraph);
      spyOn(viz, "createAxisX").andReturn(dummyAxisX);
      spyOn(viz, "createAxisY").andReturn(dummyAxisY);

      spyOn(dummyGraph, "render");

      viz.render(groupTs);
      expect(viz.createGraph).wasCalled();
      expect(dummyGraph.render).wasCalled();
    });

    it("should not show the Y axis", function() {
      var viz = new Visualizer($.extend({}, options, {showYAxis: false}));
      viz.render(groupTs);
      expect(viz.yAxis).toBeUndefined();
    });

    it("should not show the legend by default", function() {
      var viz = new Visualizer(options);
      viz.render(groupTs);
      expect(viz.legend).toBeUndefined();
      expect(viz.legendToggle).toBeUndefined();
      expect(viz.legendHighlight).toBeUndefined();
    });

    it("should show the legend", function() {
      $("#chartContainer").append($('<div class="legend"/>'));
      var viz = new Visualizer($.extend({}, options, {showLegend: true}));
      viz.render(groupTs);
      expect(viz.legend).toBeDefined();
      expect(viz.legendToggle).toBeDefined();
      expect(viz.legendHighlight).toBeDefined();
      $(".legend").remove();
    });

    it("should show the Y-axis with units set", function() {
      var viz = new Visualizer(_.defaults(options, {showYAxis: true}));
      spyOn(viz, 'createAxisY').andReturn(dummyAxisY);
      dummyAxisY.setUnits = jasmine.createSpy('setUnits');
      spyOn(FormatUtils, 'getFirstUnits').andReturn({
        numerators: ['cats'],
        denominators: ['pants']
      });
      viz.render(groupTs);
      expect(dummyAxisY.setUnits).wasCalled();
    });

    it("should call configure", function() {
      var viz = new Visualizer(options);
      spyOn(viz, "createGraph").andReturn(dummyGraph);
      spyOn(viz, "createAxisX").andReturn(dummyAxisX);
      spyOn(viz, "createAxisY").andReturn(dummyAxisY);

      spyOn(dummyGraph, "configure");
      spyOn(dummyGraph, "render");

      viz.render(groupTs);

      viz.update({
        width: 60,
        height: 50
      });

      expect(dummyGraph.configure).wasCalled();
      expect(dummyGraph.render).wasCalled();
    });
  });
});
